==> pkg_add_it-1.0.tar.gz: pkg_add_it-1.0/fnode.c <==
/*-
 * Copyright (c) 2008  Marin Atanasov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <dirent.h>
# include "fnode.h"

/* saves a matched pattern into a stack */
void fnode_push (FNODEPUSHARGS)
{
   fnode_t *newnode = (fnode_t *) malloc (sizeof(fnode_t));
   newnode->fname = (char *) malloc (strlen(fname) + 1);
   newnode->fpath = (char *) malloc (strlen(fpath) + 1);
   
   newnode->desc = (desc == NULL) ? NULL : (char *) malloc (strlen(desc) + 1);
   newnode->maint = (maint == NULL) ? NULL : (char *) malloc (strlen(maint) + 1);
   
   if (!newnode || !newnode->fname || !newnode->fpath) {
      perror ("malloc");
      exit (1);
   }
   
   strcpy (newnode->fname, fname);
   strcpy (newnode->fpath, fpath);
   if (desc)
      strcpy (newnode->desc, desc);
   if (maint)
      strcpy (newnode->maint, maint);
   
   newnode->next = *head;
   *head = newnode;
}

/* displays all found patterns */
int fnode_display (FNODEDISPLAYARGS)
{
   char *pager, buf[MAXNAMLEN + 1];
   int choice, i = 0;
   fnode_t *current = head;
   FILE *fp = stdout;
   
   /* if we have more entries than MAXLINESPERPAGE then we'll use a pager */
   if (numfiles > MAXLINESPERPAGE) {
      /* we'll use a temp file to output the 
       * entries and then read them with a pager */
      if (!(fp = fopen(PAGERTMPFILE, "w"))) {
         perror ("fopen");
         exit (1);
      }
      /* use pager defined by $PAGER, otherwise use DEFAULTPAGER */
      if (!(pager = getenv("PAGER"))) {
         pager = (char *) malloc (strlen(DEFAULTPAGER) + 1);
         strcpy (pager, DEFAULTPAGER);
      }
   }
   
   fprintf (fp, "==>> Total packages found %d\n", numfiles);
   fprintf (fp, "==>> List of packages matching the given pattern... \n\n%s", LONGLINE);
   for ( ; i < numfiles; i++, current = current->next) {
      fprintf (fp, "(%.3d)\t%-30s (in %-s)\n", i + 1, current->fname, current->fpath);
      if (current->desc)
         fprintf (fp, "\tDescription: %s\n", current->desc);
      if (current->maint)
         fprintf (fp, "\tMaintained by: %s\n\n", current->maint);
   }
   
   /* if we are using a pager */   
   if (numfiles > MAXLINESPERPAGE) {
      snprintf (buf, sizeof(buf), "%s %s", pager, PAGERTMPFILE);
      fclose (fp);
      system (buf);
      
      /* remove the tmp file */
      remove (PAGERTMPFILE);
   }
   
   fprintf (stdout, "%s\n==>> Total packages found: %d\n", LONGLINE, numfiles);
   do {
      fprintf (stdout, "==>> Choose the package number you wish to install [1-%d]: ", numfiles);
      fgets (buf, 10, stdin);
      choice = atoi (buf);
   } while (choice < 1 || choice > numfiles);
   
   return choice;
}

/* locates a node using an index number */
void fnode_locate (FNODELOCATEARGS)
{
   int i = 1;
   fnode_t *current = head;
   
   for ( ; i < index; i++, current = current->next) ; 
   
   *pkgfile = current;
}

/* deallocates everything */
void fnode_shutdown (FNODESHUTDOWNARGS)
{
   fnode_t *current = head, *next = NULL;
      
   while (current) {
      next = current->next;
      free (current);
      current = next;
   }   
}

==> pkg_add_it-1.1.tar.gz: pkg_add_it-1.1/fnode.c <==
/*-
 * Copyright (c) 2008  Marin Atanasov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <dirent.h>
# include "fnode.h"

/* saves a matched pattern into a stack */
void fnode_push (FNODEPUSHARGS)
{
   fnode_t *newnode = (fnode_t *) malloc (sizeof(fnode_t));
   newnode->fname = (char *) malloc (strlen(fname) + 1);
   newnode->fpath = (char *) malloc (strlen(fpath) + 1);
   
   newnode->desc = (desc == NULL) ? NULL : (char *) malloc (strlen(desc) + 1);
   newnode->maint = (maint == NULL) ? NULL : (char *) malloc (strlen(maint) + 1);
   
   if (!newnode || !newnode->fname || !newnode->fpath) {
      perror ("malloc");
      exit (1);
   }
   
   strcpy (newnode->fname, fname);
   strcpy (newnode->fpath, fpath);
   if (desc)
      strcpy (newnode->desc, desc);
   if (maint)
      strcpy (newnode->maint, maint);
   
   newnode->next = *head;
   *head = newnode;
}

/* displays all found patterns */
void fnode_display (FNODEDISPLAYARGS)
{
   char *pager, buf[MAXNAMLEN + 1];
   int i = 0;
   fnode_t *current = head;
   FILE *fp = stdout;
   
   /* if we have more entries than MAXLINESPERPAGE then we'll use a pager */
   if (numfiles > MAXLINESPERPAGE) {
      /* we'll use a temp file to output the 
       * entries and then read them with a pager */
      if (!(fp = fopen(PAGERTMPFILE, "w"))) {
         perror ("fopen");
         exit (1);
      }
      /* use pager defined by $PAGER, otherwise use DEFAULTPAGER */
      if (!(pager = getenv("PAGER"))) {
         pager = (char *) malloc (strlen(DEFAULTPAGER) + 1);
         strcpy (pager, DEFAULTPAGER);
      }
   }
   
   fprintf (fp, "==>> Total packages found: %d\n", numfiles);
   fprintf (fp, "==>> List of packages matching the given pattern... \n\n%s", LONGLINE);
   for ( ; i < numfiles; i++, current = current->next) {
      fprintf (fp, "(%.3d)\t%-30s (in %-s)\n", i + 1, current->fname, current->fpath);
      if (current->desc)
         fprintf (fp, "\tDescription: %s\n", current->desc);
      if (current->maint)
         fprintf (fp, "\tMaintained by: %s\n\n", current->maint);
   }
   
   /* if we are using a pager */   
   if (numfiles > MAXLINESPERPAGE) {
      snprintf (buf, sizeof(buf), "%s %s", pager, PAGERTMPFILE);
      fclose (fp);
      system (buf);
      
      /* remove the tmp file */
      remove (PAGERTMPFILE);
   }
   
   fprintf (stdout, "%s\n==>> Total packages found: %d\n", LONGLINE, numfiles);
}

/* asks the user to choose a package */
int fnode_choose_pkg (FNODECHOOSEPKGARGS)
{
   int choice;
   char buf[MAXNAMLEN +1];
   
   do {
      fprintf (stdout, "==>> Choose the package number you wish to install [1-%d]: ", numfiles);
      fgets (buf, 10, stdin);
      choice = atoi (buf);
   } while (choice < 1 || choice > numfiles);
   
   return choice;
}

/* locates a node using an index number */
void fnode_locate (FNODELOCATEARGS)
{
   int i = 1;
   fnode_t *current = head;
   
   for ( ; i < index; i++, current = current->next) ; 
   
   *pkgfile = current;
}

/* deallocates everything */
void fnode_shutdown (FNODESHUTDOWNARGS)
{
   fnode_t *current = head, *next = NULL;
      
   while (current) {
      next = current->next;
      free (current);
      current = next;
   }   
}
